require 'rubygems'
require 'json'
require 'socket'
require 'rails'
require 'agent/am_objectholder'

module ManageEngine
	class APMConfig
		attr_reader :agenthost,:agentport,:instance_id,:alreadyconnected,:apmhost,:apmport,:license_key,:site24x7
		attr_reader :appname,:proxyneeded, :apdex_t, :trans_trace, :trans_trace_t, :sql_capture, :sql_capture_params, :sql_trace_t,:proxy_user,:proxy_pass, :metric_overflow_t, :trace_overflow_t
		attr_reader :proxy_host,:proxy_port ,:is_secured, :logs_dir ,:connection_retry,:agent_enabled,:connect_interval,:db_operations,:include_packages, :url_merge_pattern
		attr_accessor :app_db,:app_dispatcher,:lastupdatedtime
		def initialize
			@obj = ManageEngine::APMObjectHolder.instance
			
    		#@config = @obj.util.readProperties(@obj.constants.apm_conf)
			configureFile
			@agenthost = Socket.gethostname
			assignConfig
			@obj.log.setLevel @config["apminsight.log.level"]
			@instance_id  = 0
			@agent_enabled = false
			@alreadyconnected = checkAgentInfo
			@site24x7 = checkLicenseFile
			@db_operations =["select","insert","update","delete"]
			urlMergePattern
			@obj.log.debug "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
			@obj.log.debug "APP HOME #{File.absolute_path(".")} "
			@obj.log.debug "APP HOME #{Dir.pwd} "
			@obj.log.debug "Configuration : "
			@obj.log.debug "Agent Name : #{@agenthost}"
			@obj.log.debug "Agent Already Connected : #{@alreadyconnected}"
			@obj.log.debug "Agent Enabled : #{@agent_enabled}"
			@obj.log.debug "Allowed DB Operations : #{@db_operations}"
			@config.each do|key,val|
			@obj.log.info "#{key} => #{val}"
				end
			@obj.log.info "URL Merge Patterns"
			@url_merge_pattern.each do |key, val|
				@obj.log.info "#{key} => #{val}"
			end
			@obj.log.debug "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
			@app_db="dummydb"
			@app_dispatcher = getDispatcher
			@lastupdatedtime=File.mtime(@obj.constants.apm_conf).to_i
 		end
		
		def configureFile
			begin
				if(FileTest.exists?(@obj.constants.apm_conf))
					#conf file exists in APPlication Home
						@obj.log.debug "Config File Exists. It is read from #{@obj.constants.apm_conf}"
					@config = @obj.util.readProperties(@obj.constants.apm_conf)
				else
					gem_conf = Gem.loaded_specs[@obj.constants.apm_gem].full_gem_path
					#gem_conf = File.join(gem_conf, 'lib')
					gem_conf = File.join(gem_conf, 'conf')
					gem_conf = File.join(gem_conf, 'apminsight.conf')
					#conf file not exists in APPlications Home. So 1. copy it for gem locations
					if @obj.util.copyFiles gem_conf,@obj.constants.apm_conf
						#copied sucessfully
						@obj.log.debug "Config File copied. It is read from #{@obj.constants.apm_conf}"
						@config = @obj.util.readProperties(@obj.constants.apm_conf)
					else
						#Problem in copying, so reading props from Conf file in Gem Location
						@obj.log.debug "Config File not copied. It is read from #{gem_conf}"
						@config = @obj.util.readProperties(gem_conf)
					end
				end
				
			rescue Exception=>e
				@obj.log.info "[Exception] Problem in Reading Configuration File : \n File : #{@obj.constants.apm_conf}"
				@obj.log.logException "#{e.message}",e
				@config = @obj.util.readProperties(gem_conf)
			ensure
			end
		end

		def checkAgentInfo
			if FileTest.exists?(@obj.constants.agent_conf)
				@obj.log.debug "Status : Agent Already Connected"
				props = @obj.util.readProperties(@obj.constants.agent_conf)
				@instance_id = props["agent.id"]
				@agent_enabled= @obj.util.getBooleanValue props["agent.enabled"]
				true
			else
				@obj.log.info "Status : Agent not connected"
				false
			end
		end

		def checkLicenseFile
			if(@license_key != nil)
				@obj.log.info "License key is not null. Connecting to site24x7"
				@obj.constants.setLicenseKey @license_key
				true
			else
				@obj.log.info "License key is null. Connecting to APPManager"
				false
			end

		end
		
		def urlMergePattern
			@url_merge_pattern = Hash.new
			begin
				if (FileTest.exists?(@obj.constants.mergepattern_conf))
					@url_merge_pattern=@obj.util.readProperties(@obj.constants.mergepattern_conf)
				end
			rescue Exception => e
				@obj.log.info "[Exception] Problem in Reading Configuration File : \n File : #{@obj.constants.mergepattern_conf}"
				@obj.log.logException "#{e.message}",e
			end
		end

		def updateAgentInfoFile(props)
			@instance_id = props["agent.id"]
			@agent_enabled= @obj.util.getBooleanValue props["agent.enabled"]
			@obj.util.writeProperties(@obj.constants.agent_conf,props)
		end

		def initValues
			@apmport=8080
			@appname="My Application"
			@proxyneeded = false
			@proxy_host="localhost"
			@proxy_port=80
			@proxy_user=""
			@proxy_pass=""
			@is_secured=false
			@logs_dir="./log"
			@connection_retry = 0
			@connect_interval = 60
			@apdex_t=0.5
			@include_packages=Array.new
			@trans_trace_t=2
			@sql_trace_t=3
			@metric_overflow_t=250
			@trace_overflow_t=60
		end

		def assignConfig
		initValues
		 @config.each do |key,value|
		 	 value = checkAndGetEnvValue(value)
			 case key
			 when "application.name" then @appname=value
			 when "apm.host" then @apmhost=value
			 when "apm.port" then @apmport=isInteger(@apmport,value)
			 when "license.key" then @license_key=value
			 when "behind.proxy" then @proxyneeded=@obj.util.getBooleanValue value
			 when "agent.server.port" then @agentport=isInteger(@agentport,value)
			 when "apdex.threshold" then @apdex_t=isFloat(@apdex_t,value)
			 when "transaction.trace.enabled" then @trans_trace=@obj.util.getBooleanValue value
			 when "transaction.trace.threshold" then @trans_trace_t=isFloat(@trans_trace_t,value)
			 when "sql.capture.enabled" then @sql_capture=@obj.util.getBooleanValue value
			 when "transaction.trace.sql.parametrize" then @sql_capture_params=@obj.util.getBooleanValue value
			 when "transaction.trace.sql.stacktrace.threshold" then @sql_trace_t=isFloat(@sql_trace_t,value)
			 when "proxy.server.host" then @proxy_host=value
			 when "proxy.server.port" then @proxy_port=isInteger(@proxy_port,value)
			 when "proxy.auth.username" then @proxy_user=value
			 when "proxy.auth.password" then @proxy_pass=value
			 when "apm.protocol.https" then @is_secured=@obj.util.getBooleanValue value
			 when "apminsight.log.dir" then @logs_dir=value
			 when "agent.connection.retry" then @connection_retry=value #Not in Conf - yet to come
			when "agent.connection.interval" then	 @connect_interval=value#Not in Conf - yet to come
			when "include.packages" then @include_packages=@obj.util.getArray value,","
			end
		 end
		end
		
		#Checks whether the given value is Environment Variable
		def checkAndGetEnvValue(data)
			value = "#{data}"[/{(.*)}/,1]
			if (value != nil && ENV.has_key?(value))
				return ENV[value]
			else
				return data
			end
		end

		def getAgentInfo
			data =  Hash.new
			agentdata = Hash.new
			agentdata = {"application.type"=>"RUBY","application.name"=>@appname,"hostname"=>@agenthost,"port"=>@agentport,"agent.version"=>"1.1"}
			data["agent_info"]=agentdata
			data["environment"]=getEnvData
			data["custom_config_info"]=getAgentConfigData
			data
		end

		def getEnvData
			env =  Hash.new
			begin
			ENV.to_hash.each do |key, value|
				env[key] = value
			end
			env["Application Path"]="#{Rails.root}"
			rescue Exception=>e
			end
			env
		end
			
		def getAgentConfigData
			agentconfig = Hash.new
			agentconfig["last.modified.time"]=@lastupdatedtime*1000
			agentconfig["apdex.threshold"]=@apdex_t
			agentconfig["sql.capture.enabled"]=0
			if @sql_capture
				agentconfig["sql.capture.enabled"]=1
			end
			agentconfig["transaction.trace.enabled"]=0
			if @trans_trace
				agentconfig["transaction.trace.enabled"]=1
			end
			agentconfig["transaction.trace.threshold"]=@trans_trace_t
			agentconfig["transaction.trace.sql.parametrize"]=0
			if @sql_capture_params
				agentconfig["transaction.trace.sql.parametrize"]=1
			end
			agentconfig["transaction.trace.sql.stacktrace.threshold"]=@sql_trace_t
			agentconfig["transaction.tracking.request.interval"]=1
			agentconfig
		end

		def getDispatcher
		dispatcher = "unknown"
		if defined?(PhusionPassenger) then
			dispatcher = "passenger"
		end
		dispatcher
		end
		
		def isInteger default,value
			if @obj.util.is_integer value
				value.to_i
			else
				@obj.log.info "Problem in getting Integer Value #{value} .. So setting default value #{default} "
				default.to_i
			end

		end

		def isFloat default,value
			if @obj.util.is_float value
				value.to_f
			else
				default.to_f
				@obj.log.info "Problem in getting Integer Value #{value} .. So setting default value #{default} "
			end
		end

		def update_config configInfo
			filepath = @obj.constants.apm_conf
			f = "apminsight.conf.new"
			begin
				propsFile=File.open(filepath, 'r')
				file = File.new(f,"w+")
	      			propsFile.read.each_line do |line|
        			line.strip!
        			if (line[0] != ?# and line[0] != ?=)
          				i = line.index('=')
        	  			if (i)
						key1 = line[0..i - 1].strip
						if configInfo.has_key?(key1)
							file.puts "#{key1}=#{configInfo[key1]}\n"
						else
							file.puts "#{line}\n"
						end
	          			else
						file.puts "#{line}\n"
         				end
				else
					file.puts "#{line}\n"
        			end
      	  		end
			rescue Exception=>e
				@obj.log.info "Problem in Reading / Writing Property File :  #{e.message} "
				@obj.log.error "#{e.backtrace}"
			ensure
					propsFile.close
					file.close
			end
			res = @obj.util.copyFiles f, filepath
			if res
				@obj.log.info "copyFiles result = #{res}"
				#delete apminsight.conf.new has to be done
			end
			configureFile
			assignConfig
		end


	end#c
end#m

